/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 *
 * This file is part of de.bsvrz.pua.prot.
 *
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.pua.prot.aggregations;

import de.bsvrz.pua.prot.util.ResultValue;

/**
 * Basis Interface für alle Klassen, die Aggregationen durchführen. Abgeleitete Klassen <b>müssen</b> einen Leerkonstruktor<br> (<code>public
 * Aggregation()</code>)<br> anbieten!
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public interface Aggregation {

	/**
	 * Führt die Aggregierung durch.
	 *
	 * @param value Wert
	 *
	 * @throws UnsupportedOperationException Operation kann auf den Daten nicht durchgeführt werden.
	 */
	public void aggregate(ResultValue value) throws UnsupportedOperationException;

	/**
	 * Liefert das Ergebnis der Aggregation. Der Aufruf dieser Methode kennzeichnet einen neuen Aggregationszyklus.
	 *
	 * @return Aggregierter Wert.
	 *
	 * @throws UnsupportedOperationException Operation kann auf den Daten nicht durchgeführt werden.
	 */
	public ResultValue getResult() throws UnsupportedOperationException;

	/**
	 * Liefert den Namen der Aggregation zurück.
	 *
	 * @return Name der Aggregation
	 */
	public String getAggregationName();
}
	
