/*
 * Copyright 2019 by Kappich Systemberatung Aachen
 *
 * This file is part of de.bsvrz.pua.prot.
 *
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.pua.prot.manager.taskmanager;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.manager.datamanager.DataManagerInterface;
import de.bsvrz.pua.prot.processing.ProcessingInterface;
import de.bsvrz.pua.prot.sender.PuaSender;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

/**
 * Basisklasse für Tasks, die Protokolle erstellen/abrufen
 *
 * @author Kappich Systemberatung
 */
public abstract class AbstractTask {
	private final ClientDavInterface _dav;
	private final SystemObject _client;
	private final long _requestId;
	private final long _protocolId;
	private final long _jobId;
	private final long _startTime;
	private final long _initialFlowControl;
	
	protected static final Debug _debug = Debug.getLogger();

	public AbstractTask(final ClientDavInterface dav, final SystemObject client, final long requestId, final long protocolId, final long jobId, final long startTime, final long initialFlowControl) {
		_dav = dav;
		_client = client;
		_requestId = requestId;
		_protocolId = protocolId;
		_jobId = jobId;
		_startTime = startTime;
		_initialFlowControl = initialFlowControl;
	}

	public ClientDavInterface getDav() {
		return _dav;
	}

	public SystemObject getClient() {
		return _client;
	}

	public long getRequestId() {
		return _requestId;
	}

	public long getProtocolId() {
		return _protocolId;
	}

	public long getJobId() {
		return _jobId;
	}

	public long getStartTime() {
		return _startTime;
	}

	public long getInitialFlowControl() {
		return _initialFlowControl;
	}

	public abstract ThreadElement.Operation getOperation();

	/**
	 * Führt die Datenaufbereitung durch.
	 *
	 * @return Ergebnis der Datenaufbereitung.
	 *
	 * @throws FailureException Metainformation des Skripts ist korrupt.
	 * @param puaSender        Sender für Antworten
	 * @param dataManager      Datenverwaltung
	 */
	protected abstract ProcessingInterface process(DataManagerInterface dataManager, ThreadElement threadElement, final PuaSender puaSender) throws FailureException;

	@Override
	public String toString() {
		return getClass().getSimpleName() + "{" +
				"Anfrage-ID=" + _requestId +
				", Protokoll-ID=" + _protocolId +
				", Auftrags-ID=" + _jobId +
				'}';
	}
}
