/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 *
 * This file is part of de.bsvrz.pua.prot.
 *
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.pua.prot.data;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
abstract class AbstractArrayView implements Data.Array, Data.TextArray, Data.TimeArray,  Data.ReferenceArray, Data.NumberArray{

	// Zugriffe auf einzelne Elemente
	
	public final Data.TextValue getTextValue(final int itemIndex) {
		return getItem(itemIndex).asTextValue();
	}

	public abstract Data.NumberValue getValue(int itemIndex);

	public final Data.TimeValue getTimeValue(final int itemIndex) {
		return getItem(itemIndex).asTimeValue();
	}

	public final Data.NumberValue getUnscaledValue(final int itemIndex) {
		return getItem(itemIndex).asUnscaledValue();
	}

	public final Data.NumberValue getScaledValue(final int itemIndex) {
		return getItem(itemIndex).asScaledValue();
	}

	public final Data.ReferenceValue getReferenceValue(final int itemIndex) {
		return getItem(itemIndex).asReferenceValue();
	}

	// Zugriffe auf einzelne Elemente, Standard-Datentypen

	public final double doubleValue(final int itemIndex) {
		return getValue(itemIndex).doubleValue();
	}

	public final float floatValue(final int itemIndex) {
		return getValue(itemIndex).floatValue();
	}

	public final long longValue(final int itemIndex) {
		return getValue(itemIndex).longValue();
	}

	public final int intValue(final int itemIndex) {
		return getValue(itemIndex).intValue();
	}

	public final short shortValue(final int itemIndex) {
		return getValue(itemIndex).shortValue();
	}

	public final byte byteValue(final int itemIndex) {
		return getValue(itemIndex).byteValue();
	}

	public final SystemObject getSystemObject(final int itemIndex) {
		return getReferenceValue(itemIndex).getSystemObject();
	}

	public final String getText(final int itemIndex) {
		return getTextValue(itemIndex).getText();
	}

	public final long getSeconds(final int itemIndex) {
		return getTimeValue(itemIndex).getSeconds();
	}

	public final long getMillis(final int itemIndex) {
		return getTimeValue(itemIndex).getMillis();
	}

	// Array-Konvertierungen

	public final Data.TextArray asTextArray() {
		return this;
	}

	public final Data.TimeArray asTimeArray() {
		return this;
	}

	public final Data.ReferenceArray asReferenceArray() {
		return this;
	}

	public Data.NumberArray asScaledArray() {
		return this;
	}

	// Wert-Array-Funktionen

	public final Data.TextValue[] getTextValues() {
		final Data.TextValue[] textValues = new Data.TextValue[getLength()];
		for(int i = 0; i < textValues.length; i++) {
			textValues[i] = getItem(i).asTextValue();
		}
		return textValues;
	}

	public final Data.TimeValue[] getTimeValues() {
		final Data.TimeValue[] timeValues = new Data.TimeValue[getLength()];
		for(int i = 0; i < timeValues.length; i++) {
			timeValues[i] = getItem(i).asTimeValue();
		}
		return timeValues;
	}

	public final Data.NumberValue[] getScaledValues() {
		final Data.NumberValue[] numberValues = new Data.NumberValue[getLength()];
		for(int i = 0; i < numberValues.length; i++) {
			numberValues[i] = getItem(i).asScaledValue();
		}
		return numberValues;
	}

	public final Data.NumberValue[] getUnscaledValues() {
		final Data.NumberValue[] numberValues = new Data.NumberValue[getLength()];
		for(int i = 0; i < numberValues.length; i++) {
			numberValues[i] = getItem(i).asUnscaledValue();
		}
		return numberValues;
	}

	public final Data.NumberValue[] getValues() {
		final Data.NumberValue[] numberValues = new Data.NumberValue[getLength()];
		for(int i = 0; i < numberValues.length; i++) {
			numberValues[i] = getValue(i);
		}
		return numberValues;
	}

	public final Data.ReferenceValue[] getReferenceValues() {
		final Data.ReferenceValue[] referenceValues = new Data.ReferenceValue[getLength()];
		for(int i = 0; i < referenceValues.length; i++) {
			referenceValues[i] = getItem(i).asReferenceValue();
		}
		return referenceValues;
	}

	// Diese Funktionen müssten evtl. die "Berechtigungen" aus dem Attribut zurückliefern, da die Werte aber eh nicht änderbar sind, erscheint das nicht zwingend erforderlich

	public final boolean isCountVariable() {
		return false;
	}

	public final boolean isCountLimited() {
		return true;
	}

	// Direkte Array-Konvertierungen

	public final long[] getSecondsArray() {
		final Data.TimeValue[] timeValues = getTimeValues();
		final long[] longs = new long[timeValues.length];
		for(int i = 0; i < timeValues.length; i++) {
			longs[i] = timeValues[i].getSeconds();
		}
		return longs;
	}

	public final long[] getMillisArray() {
		final Data.TimeValue[] timeValues = getTimeValues();
		final long[] longs = new long[timeValues.length];
		for(int i = 0; i < timeValues.length; i++) {
			longs[i] = timeValues[i].getMillis();
		}
		return longs;
	}

	public final double[] getDoubleArray() {
		final Data.NumberValue[] numberValues = getValues();
		final double[] doubles = new double[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			doubles[i] = numberValues[i].doubleValue();
		}
		return doubles;
	}

	public final float[] getFloatArray() {
		final Data.NumberValue[] numberValues = getValues();
		final float[] floats = new float[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			floats[i] = numberValues[i].floatValue();
		}
		return floats;
	}

	public final long[] getLongArray() {
		final Data.NumberValue[] numberValues = getValues();
		final long[] longs = new long[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			longs[i] = numberValues[i].longValue();
		}
		return longs;
	}

	public final int[] getIntArray() {
		final Data.NumberValue[] numberValues = getValues();
		final int[] ints = new int[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			ints[i] = numberValues[i].intValue();
		}
		return ints;
	}

	public final short[] getShortArray() {
		final Data.NumberValue[] numberValues = getValues();
		final short[] shorts = new short[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			shorts[i] = numberValues[i].shortValue();
		}
		return shorts;
	}

	public final byte[] getByteArray() {
		final Data.NumberValue[] numberValues = getValues();
		final byte[] bytes = new byte[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			bytes[i] = numberValues[i].byteValue();
		}
		return bytes;
	}

	public final SystemObject[] getSystemObjectArray() {
		final Data.ReferenceValue[] numberValues = getReferenceValues();
		final SystemObject[] systemObjects = new SystemObject[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			systemObjects[i] = numberValues[i].getSystemObject();
		}
		return systemObjects;
	}

	public final String[] getTextArray() {
		final Data.TextValue[] numberValues = getTextValues();
		final String[] strings = new String[numberValues.length];
		for(int i = 0; i < numberValues.length; i++) {
			strings[i] = numberValues[i].getText();
		}
		return strings;
	}

	// Set-Funktionen

	public final void set(final double... doubles) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final float... floats) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final long... longs) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final int... ints) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final short... shorts) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final byte... bytes) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final SystemObject... systemObjects) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final ObjectLookup dataModel, final String... systemObjectPids) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void set(final String... strings) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void setMillis(final long... millis) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void setSeconds(final long... seconds) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public final void setLength(final int newLength) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}
}
