/*
 * Copyright 2018 by Kappich Systemberatung Aachen
 *
 * This file is part of de.bsvrz.pua.prot.
 *
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * Kappich Systemberatung
 * Martin-Luther-Straße 14
 * 52062 Aachen, Germany
 * phone: +49 241 4090 436
 * mail: <info@kappich.de>
 */

package de.bsvrz.pua.prot.util;

import com.google.common.collect.ImmutableMap;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.kappich.annotations.NotNull;
import de.bsvrz.sys.funclib.kappich.annotations.Nullable;
import de.bsvrz.sys.funclib.losb.datk.NoChangeMarker;
import de.bsvrz.sys.funclib.losb.datk.ProtocolType;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;

public class ProcessingParameterBuilder {
	@NotNull
	private final SystemObject _script;
	@Nullable
	private SystemObject _mainObject = null;
	
	private short _simVar = (short) 0;
	@NotNull
	private PeriodSet _periods = PeriodSet.of();
	@NotNull
	private Map<String, String> _aspBindings = ImmutableMap.of();
	@NotNull
	private Map<String, List<String>> _pseudoObjects = ImmutableMap.of();
	
	private boolean _saveProtocol = false;
	@NotNull
	private ProtocolType _protocolType = ProtocolType.Undefined;
	@NotNull
	private NoChangeMarker _noChangeMarker = NoChangeMarker.Row;
	@NotNull
	private Collection<ArchiveDataKind> _archiveDataKinds = ArchiveDataKindCombination.all();
	@NotNull
	private String _creatorName = "";
	
	private boolean _isPublic = true;
	@NotNull
	private String _infoText = "";

	public ProcessingParameterBuilder(final SystemObject script) {
		_script = Objects.requireNonNull(script);
	}

	public ProcessingParameterBuilder withMainObject(final SystemObject mainObject) {
		_mainObject = mainObject;
		return this;
	}

	public ProcessingParameterBuilder withSimVar(final short simVar) {
		_simVar = simVar;
		return this;
	}

	public ProcessingParameterBuilder withPeriods(final PeriodSet periods) {
		_periods = periods;
		return this;
	}

	public ProcessingParameterBuilder withAspBindings(final Map<String, String> aspBindings) {
		_aspBindings = Objects.requireNonNull(aspBindings);
		return this;
	}

	public ProcessingParameterBuilder withPseudoObjects(final Map<String, List<String>> pseudoObjects) {
		_pseudoObjects = Objects.requireNonNull(pseudoObjects);
		return this;
	}

	public ProcessingParameterBuilder withSaveProtocol(final boolean saveProtocol) {
		_saveProtocol = saveProtocol;
		return this;
	}

	public ProcessingParameterBuilder withProtocolType(final ProtocolType protocolType) {
		_protocolType = Objects.requireNonNull(protocolType);
		return this;
	}

	public ProcessingParameterBuilder withNoChangeMarker(final NoChangeMarker noChangeMarker) {
		_noChangeMarker = Objects.requireNonNull(noChangeMarker);
		return this;
	}

	public ProcessingParameterBuilder withArchiveDataKinds(final Collection<ArchiveDataKind> archiveDataKind) {
		_archiveDataKinds = Objects.requireNonNull(archiveDataKind);
		return this;
	}

	public ProcessingParameterBuilder withCreatorName(final String creatorName) {
		_creatorName = Objects.requireNonNull(creatorName);
		return this;
	}

	public ProcessingParameterBuilder withIsPublic(final boolean isPublic) {
		_isPublic = isPublic;
		return this;
	}

	public ProcessingParameterBuilder withInfoText(final String infoText) {
		_infoText = Objects.requireNonNull(infoText);
		return this;
	}

	public ProcessingParameter build() {
		try {
			return new ProcessingParameter(_script, _mainObject, _simVar, _periods, _aspBindings, _pseudoObjects, _saveProtocol, _protocolType, _noChangeMarker, _archiveDataKinds, _creatorName, _isPublic, _infoText);
		}
		catch(FailureException e) {
			throw new IllegalArgumentException(e);
		}
	}
}
